﻿<#
  .Synopsis
   Clear the Windows Update Cache.

  .Description
   Clear the Windows Update Cache. Provides appropriate access
   checks, stops Window Update service and restarts it upon
   completion. 

  .Parameter Setup
   If present the program will create a scheduled task with
   administrator privleges and create a desktop shortcut to
   run the task.

  .Outputs
   Status messages. 

  .Notes
     Programmer   : RetiredGeek (@askWoody.com) aka: ComputerMentor
     Created      : 08 Mar 2020
     Last Updated : 
     Current Vers : 1.0

  .Example
   [d:\path\]Clear-WindowsUpdateCache.ps1

   Run the program and clear the cache.

  .Example
   [d:\path\]Clear-WindowsUpdateCache.ps1 -Setup

   Create a scheduled task and short to run it.

#>


# Two blank linea above required for Comment-Based Help#Clear Windows Update Cache

Param
(
   [Parameter(Mandatory=$false)]
     [Switch] $Setup
)

Function Get-AdminStatus {
<#
  .Synopsis
    Determine if PowerShell is running with Administrator
    privleges.
 
  .Outputs
    [ User | Administrator ] as String

  .Notes
     Programmer   : RetiredGeek (WSL) aka: ComputerMentor
     Created      : 08 Jul 2016
     Last Updated : 
     Current Vers : 1.0

     If role is ANYTHING other than Administrator the code
     will return USER!

  .Example
     $Priv = Get-AdminStatus

  .Example
     If ((Get-AdminStatus) -eq "User") {…} Else {…}

#>
$identity = 
   [Security.Principal.WindowsIdentity]::GetCurrent()
 $principal = 
   New-Object Security.Principal.WindowsPrincipal $identity

If (-NOT ($principal.IsInRole(
    [Security.Principal.WindowsBuiltinRole]::Administrator)))
  {"User"}
Else
  {"Administrator"}

} # -----------  End Function Get-AdminStatus -----------------

Function Install-Program {

  #--- Create Scheduled Task to runas Administrator   ---
  $TaskSet = New-ScheduledTaskSettingsSet -Compatibility Win8 
  $TaskSet.DisallowStartIfOnBatteries = $False
  $TaskSet.StopIfGoingOnBatteries     = $False
  $TaskSet.IdleSettings.StopOnIdleEnd = $False
 
  $TaskCommand = "c:\windows\system32\windowspowershell" +
                     "\v1.0\powershell.exe" 
  $TaskArgs    = "-WindowStyle Minimized -Command " + 
                 '"' + $PSCommandPath + '"'

  $TAArgs = @{Execute  = '"' + $TaskCommand + '"'
              Argument =   $TaskArgs 
             }
  
  $TaskAction = New-ScheduledTaskAction @TAArgs
  
  $Parts = $PSCommandPath.Split('\')
  $FileName = $($Parts[($Parts.count)-1].Split('.'))[0]
  
  $TaskDesc    = "Run $FileName as Administrator"
  $TaskName    = "$FileName as Administrator"

  $RSTArgs = @{Action      = $TaskAction  
               TaskName    = "$TaskName" 
               User        = "$env:USERDOMAIN\$env:username" 
               RunLevel    = "Highest" 
               Description = "$TaskDesc" 
               Settings    = $TaskSet}

  Register-ScheduledTask @RSTargs -Force


  #--- Create Desktop Shortcut to call Scheduled Task ---

  $ShortcutName    = "$TaskName" 
  $DestinationPath = "C:\Users\$env:Username\Desktop"
  $ShortcutCommand = "C:\Windows\System32\schtasks.exe " +
                     "/run /TN " + '"'  + $TaskName + '"'
 
  $WshShell = New-Object -comObject WScript.Shell
  $Shortcut = $WshShell.CreateShortcut(
              "$DestinationPath\$ShortcutName.lnk")
  $Shortcut.TargetPath = "C:\Windows\System32\schtasks.exe" 
  $Shortcut.Arguments = "`/run `/TN " + '"' + $ShortcutName + '"'
  $Shortcut.WindowStyle = 7
  $Shortcut.IconLocation = $TaskCommand

  
  $Shortcut.Save()

} #End Install-Program

#-----------------   Main Program   ---------------------------

$PGMVers = 1.0
$PGMName = "Clear-WindowsUpdateCache"
Add-Type -AssemblyName System.Windows.Forms

$MsgBox  = [Windows.Forms.MessageBox]
$Buttons = [Windows.Forms.MessageBoxButtons]
#Values: OK, OkCancel, AbortRetryIgnore, YesNo, 
#            YesNoCancel, RetryCancel

$Icon    = [Windows.Forms.MessageBoxIcon]
#Values: None Question, [Stop | Hand], 
#        [Warning | Exclamation], [Information | Asterisk]

$top = 
 new-Object System.Windows.Forms.Form -property @{Topmost=$true}
$StatusMsg = {
  $MsgBox::Show($top,$Message,$MBTitle,$Buttons,$Icon)}

$MBTitle="$PGMName Vers. $PGMVers - Program Terminated"
$Buttons="OK"
$Icon="Stop"

If ((Get-AdminStatus) -eq "User") {
  $Message = "Program requires Administrator privlidges!`n" +
             "Please rerun AsAdmin..."
  $Null    = & $StatusMsg
  Exit
}

If ($Setup.IsPresent) {
  Install-Program
  $Message = "Program installed successfully.`n`n" +
             "Scheduled task and shortcut created."
  $Icon    = "Information"
  $MBTitle = "$PGMName Vers. $PGMVers - Installation Successful:"
  $Null    = & $StatusMsg
  Exit
}

Try {
      Stop-Service -Name "wuauserv" -Force -EA Stop
}
Catch {
  $Message = "The Windows Update Service failed to stop!"
  $Null    = & $StatusMsg
  Exit
}

Try {
     $UPDPath = "C:\Windows\SoftwareDistribution\Download"

     GCI -Path "$UPDPath" | 
        Remove-Item -Recurse -Force
}
Catch {
  $Message = "Cleanup of $($RIArgs.Path) Failed!"
  $Null    = & $StatusMsg
  Exit 
}

Try {
  Start-Service -Name "wuauserv" -EA Stop
  $Message = "The Windows Update Cache cleared successfully!"
  $MBTitle = "$PGMName Vers. $PGMVers - " +
             "Program completed Successfully:"
  $Icon    = "Information"
  $Null    = & $StatusMsg

}
Catch {
  $Message = "The Windows Update Service failed to restart!"
  $MBTitle = "$PGMName Vers. $PGMVers - " +
              "Program completed with errors!:"
  $Icon    = "Warning"
  $Null    = & $StatusMsg
}